//@author: vux
//@help: standard constant shader
//@tags: color
//@credits: 

cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : VIEWPROJECTION;
	float4x4 tW : WORLD;
	float4x4 tWVP: WORLDVIEWPROJECTION;
	float4x4 tVI : VIEWINVERSE;
}

Texture2D texture2dSPRITE <string uiname="texture2dSPRITE";>;
Texture2D texture2dXYZ <string uiname="TextureXYZ";>;
float alpha;
float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
float4x4 tColor <string uiname="Color Transform";>;

SamplerState g_samLinear : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

float3 g_positions[4]: IMMUTABLE =
    {
        float3( -1, 1, 0 ),
        float3( 1, 1, 0 ),
        float3( -1, -1, 0 ),
        float3( 1, -1, 0 ),
    } ;
float2 g_texcoords[4]: IMMUTABLE = 
    { 
        float2(0,1), 
        float2(1,1),
        float2(0,0),
        float2(1,0),
    };
 
float radius = 0.01f;




struct vsIn
{
	float4 pos : POSITION;
	float4 uv : TEXCOORD0;
};

struct gsIn
{
    float4 pos: POSITION;
	float4 uv : TEXCOORD0;
}; 

struct psIn
{
	float4 pos: SV_POSITION;
    float2 uv: TEXCOORD0;
};



vsIn VS(vsIn input)
{
    
	gsIn output;
    output.pos  = texture2dXYZ.SampleLevel(g_samLinear,input.uv.xy,0);;
    output.uv = input.uv;
	return output;

}

[maxvertexcount(4)]
void GS_Particle(point gsIn input[1], inout TriangleStream<psIn> SpriteStream)
{
    psIn output;
    for(int i=0; i<4; i++)
    {
        float3 position = g_positions[i]*radius;
        position = mul( position, (float3x3)tVI ) + input[0].pos.xyz;
        output.pos = mul( float4(position,1.0), tWVP );
        output.uv = g_texcoords[i];
        SpriteStream.Append(output);
    }
    SpriteStream.RestartStrip();
}


float4 PS(psIn In): SV_Target
{
    float4 col = texture2dSPRITE.Sample(g_samLinear,In.uv) * cAmb;
	
//	col = mul(col, tColor);
	col.a = alpha;
    return col;
}





technique10 SPRITES
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS_Particle() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




